import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { Children, cloneElement, forwardRef, isValidElement, useEffect, useMemo, useRef, useState } from "react";
import { useFloating, offset, flip, shift, useListNavigation, useHover, useTypeahead, useInteractions, useRole, useClick, useDismiss, autoUpdate, safePolygon, FloatingPortal, useFloatingTree, useFloatingNodeId, useFloatingParentNodeId, FloatingNode, FloatingTree, FloatingFocusManager } from "@floating-ui/react-dom-interactions";
import cx from "classnames";
import { mergeRefs } from "react-merge-refs";
import { useTheme } from "./Theme/Providers";
import { Menu2Item } from "./Menu";
/*直接改造 源自官方例子的： https://codesandbox.io/s/admiring-lamport-5wt3yg?file=/src/DropdownMenu.tsx
样式直接文件注入的形式：
原来例子 import "./styles.css";
* */
/*菜单项目； 必须放在DdMenu下面的；
* */
export const DdMenuItem = forwardRef(({ label, disabled, onClick, ...props }, ref) => {
    const theme = useTheme();
    const { getItemProps } = useInteractions();
    const dark = theme.colors.mode === "dark";
    return (_jsx(Menu2Item, { ...props, ref: ref, role: "menuitem", disabled: disabled, style: {
            display: 'flex',
            justifyContent: 'space-between',
            width: '100%',
            backgroundColor: 'ghostwhite',
            //background: 'transparent',       //#
            border: '1px',
            borderRadius: '4px',
            fontSize: '1rem',
            textAlign: 'left',
            lineHeight: '1.8',
            minWidth: '100px',
            margin: 0,
            outline: 0,
        }, css: {
            ":focus,:not([disabled]):active": {
                // background: 'royalblue',
                color: 'red',
                background: dark
                    ? 'green' : 'yellow'
            }
        }, ...getItemProps({
            onClick,
        }), children: label }));
});
export const MenuComponent = forwardRef(({ icon, children, label, ...props }, ref) => {
    const theme = useTheme();
    const [open, setOpen] = useState(false);
    const [activeIndex, setActiveIndex] = useState(null);
    const [allowHover, setAllowHover] = useState(false);
    const listItemsRef = useRef([]);
    const listContentRef = useRef(Children.map(children, (child) => isValidElement(child) ? child.props.label : null));
    const tree = useFloatingTree();
    const nodeId = useFloatingNodeId();
    const parentId = useFloatingParentNodeId();
    const nested = parentId != null;
    const { x, y, reference, floating, strategy, refs, context } = useFloating({
        open,
        onOpenChange: setOpen,
        middleware: [
            offset({ mainAxis: 4, alignmentAxis: nested ? -5 : 0 }),
            flip(),
            shift()
        ],
        placement: nested ? "right-start" : "bottom-start",
        nodeId,
        whileElementsMounted: autoUpdate
    });
    const { getReferenceProps, getFloatingProps, getItemProps } = useInteractions([
        useHover(context, {
            handleClose: safePolygon({ restMs: 25 }),
            enabled: nested && allowHover,
            delay: { open: 75 }
        }),
        useClick(context, {
            toggle: !nested,
            event: "mousedown",
            ignoreMouse: nested
        }),
        useRole(context, { role: "menu" }),
        useDismiss(context),
        useListNavigation(context, {
            listRef: listItemsRef,
            activeIndex,
            nested,
            onNavigate: setActiveIndex
        }),
        useTypeahead(context, {
            listRef: listContentRef,
            onMatch: open ? setActiveIndex : undefined,
            activeIndex
        })
    ]);
    // Event emitter allows you to communicate across tree components.
    // This effect closes all menus when an item gets clicked anywhere
    // in the tree.
    useEffect(() => {
        function onTreeClick() {
            setOpen(false);
            if (parentId === null) {
                refs.reference.current?.focus();
            }
        }
        tree?.events.on("click", onTreeClick);
        return () => {
            tree?.events.off("click", onTreeClick);
        };
    }, [parentId, tree, refs]);
    // Determine if "hover" logic can run based on the modality of input. This
    // prevents unwanted focus synchronization as menus open and close with
    // keyboard navigation and the cursor is resting on the menu.
    useEffect(() => {
        function onPointerMove() {
            setAllowHover(true);
        }
        function onKeyDown() {
            setAllowHover(false);
        }
        window.addEventListener("pointermove", onPointerMove, {
            once: true,
            capture: true
        });
        window.addEventListener("keydown", onKeyDown, true);
        return () => {
            window.removeEventListener("pointermove", onPointerMove, {
                capture: true
            });
            window.removeEventListener("keydown", onKeyDown, true);
        };
    }, [allowHover]);
    const mergedReferenceRef = useMemo(() => mergeRefs([ref, reference]), [
        reference,
        ref
    ]);
    return (_jsxs(FloatingNode, { id: nodeId, children: [_jsxs(Menu2Item, { variant: "ghost", ...getReferenceProps({
                    ...props,
                    ref: mergedReferenceRef,
                    onClick(event) {
                        event.stopPropagation();
                        event.currentTarget.focus();
                    },
                    ...(nested
                        ? {
                            className: cx("MenuItem", { open }),
                            role: "menuitem",
                            onKeyDown(event) {
                                // Prevent more than one menu from being open.
                                if (event.key === "ArrowUp" || event.key === "ArrowDown") {
                                    setOpen(false);
                                }
                            }
                        }
                        : {
                            className: cx("RootMenu", { open }),
                        }),
                    style: {
                        border: nested ? undefined : 'none',
                        marginLeft: nested ? 0 : undefined,
                        backgroundColor: 'white',
                        borderWidth: nested ? '1px' : 'unset',
                        borderStyle: nested ? 'dotted' : 'unset',
                        borderRadius: nested ? '7px' : 'unset',
                    },
                }), css: {
                    width: nested ? '100%' : 'unset',
                    ":hover": {
                        color: 'blue',
                    },
                }, children: [icon && icon, !icon && label, " ", !icon && nested && _jsx("span", { style: { marginLeft: 11 }, children: "\u2794" })] }), _jsx(FloatingPortal, { children: open && (_jsx(FloatingFocusManager, { context: context, modal: !nested, returnFocus: !nested, 
                    // Touch-based screen readers will be able to navigate back to the
                    // reference and click it to dismiss the menu without clicking an item.
                    // This acts as a touch-based `Esc` key. A visually-hidden dismiss button
                    // is an alternative.
                    order: ["reference", "content"], children: _jsx("div", { className: "MGroup", style: {
                            border: '1px solid #d7dce5',
                            borderRadius: '6px',
                            boxShadow: '2px 4px 12px rgba(0, 0, 0, 0.1)',
                            outline: 0,
                            width: 'max-content',
                            margin: 0,
                            display: 'flex',
                        }, ...getFloatingProps({
                            className: "Menu",
                            ref: floating,
                            style: {
                                position: strategy,
                                top: y ?? 0,
                                left: x ?? 0,
                                padding: nested ? '0 0.3rem' : '0 0',
                                zIndex: theme.zIndices.menus,
                                background: 'white', //nested? 'transparent' :
                            },
                            onKeyDown(event) {
                                if (event.key === "Tab") {
                                    setOpen(false);
                                }
                            }
                        }), css: {
                            ":hover": {
                            //    background: 'green',
                            }
                        }, children: Children.map(children, (child, index) => isValidElement(child) &&
                            cloneElement(child, getItemProps({
                                tabIndex: -1,
                                role: "menuitem",
                                className: "MenuItem",
                                ref(node) {
                                    listItemsRef.current[index] = node;
                                },
                                onClick() {
                                    console.log("Pressed! 点了");
                                    child.props.onClick && child.props.onClick();
                                    tree?.events.emit("click");
                                },
                                // By default `focusItemOnHover` uses `mousemove` to sync focus,
                                // but when a menu closes we want this to sync it on `enter`
                                // even if the cursor didn't move. NB: Safari does not sync in
                                // this case.
                                onPointerEnter() {
                                    if (allowHover) {
                                        setActiveIndex(index);
                                    }
                                }
                            }))) }) })) })] }));
});
/*可自由互动布局的 菜单； <IconMoreVertical/>
* */
export const DdMenu = forwardRef((props, ref) => {
    const parentId = useFloatingParentNodeId();
    const theme = useTheme();
    if (parentId == null) {
        return (_jsx(FloatingTree, { children: _jsx(MenuComponent, { ...props, ref: ref, style: {
                    padding: '6px 14px',
                    fontSize: '16px',
                    background: 'none',
                    borderRadius: '6px',
                    border: 'none',
                    // border: '1px solid #d7dce5',
                } }) }));
    }
    return _jsx(MenuComponent, { ...props, ref: ref, style: {
            background: 'white',
            padding: '4px',
            border: '1px solid #d7dce5',
            borderRadius: '6px',
            boxShadow: '2px 4px 12px rgba(0, 0, 0, 0.1)',
            outline: 0,
            //width: 'max-content',   嵌套的menu item class="MenuItem  "
            margin: 0,
            display: 'flex',
        } });
});
//# sourceMappingURL=DropdownMenu.js.map